/*
 * ZipFile.java
 * 2004/08/05
 * by s.s.k.
 */

import java.io.*;
import java.util.zip.*;

/**
 * @author s.s.k.
 */
public class ZipFile {

	/** 圧縮レベル (0～9) */
	private static final int COMPRESS_LEVEL = 9;


	/**
	 * ファイルをzipファイルに圧縮する。
	 * 
	 * @param files
	 *        圧縮されるファイル
	 * @param zipFile
	 *        zipファイル
	 * @throws IOException
	 *         ファイル入出力エラー
	 */
	public void zip( File[] files, File zipFile ) throws IOException{
		ZipOutputStream output = new ZipOutputStream( new FileOutputStream(
				zipFile ) );
		output.setLevel( COMPRESS_LEVEL );
		for( int i = 0; i < files.length; i++ ){
			writeEntry( files[i], output, 0 );
		}
		output.close();
	}


	/**
	 * ファイルとその階層を指定し、ZipOutputStreamにファイルを追加する。
	 * 
	 * @param file
	 *        ファイル
	 * @param output
	 *        ZipOutputStream
	 * @param depth
	 *        ファイルの階層
	 * @throws IOException
	 *         ファイル入出力エラー
	 */
	private void writeEntry( File file, ZipOutputStream output, int depth )
			throws IOException{
		if( file.isDirectory() == true ){
			File[] insideFiles = file.listFiles();
			for( int i = 0; i < insideFiles.length; i++ ){
				writeEntry( insideFiles[i], output, depth + 1 );
			}
		}else{
			BufferedInputStream input = new BufferedInputStream(
					new FileInputStream( file ) );
			file = file.getCanonicalFile();
			String fileName = file.getPath();

			int lastIndex = fileName.length();
			for( int i = 0; i <= depth; i++ ){
				lastIndex = fileName.lastIndexOf( File.separator, lastIndex ) - 1;
			}
			fileName = fileName.substring( lastIndex + 2 );

			ZipEntry entry = new ZipEntry( fileName );
			entry.setCompressedSize( getCompressedSize( file ) );
			entry.setCrc( getCRCValue( file ) );
			entry.setMethod( ZipEntry.DEFLATED );
			entry.setSize( file.length() );
			entry.setTime( file.lastModified() );
			output.putNextEntry( entry );

			int b;
			while( (b = input.read()) != -1 ){
				output.write( b );
			}
			input.close();
			output.closeEntry();
		}
	}


	/**
	 * ファイルのCRC-32チェックサムを取得する。
	 * 
	 * @param file
	 *        ファイル
	 * @return CRC-32チェックサム
	 * @throws IOException
	 *         ファイル入出力エラー
	 */
	private long getCRCValue( File file ) throws IOException{
		CRC32 crc = new CRC32();

		BufferedInputStream input = new BufferedInputStream(
				new FileInputStream( file ) );
		int b;
		while( (b = input.read()) != -1 ){
			crc.update( b );
		}
		input.close();
		return crc.getValue();
	}


	/**
	 * ファイルの圧縮後のサイズを調べる。
	 * 
	 * @param file
	 *        ファイル
	 * @return 圧縮後のサイズ
	 * @throws IOException
	 *         入出力エラー
	 */
	private long getCompressedSize( File file ) throws IOException{
		ZipEntry entry = new ZipEntry( file.getName() );
		entry.setMethod( ZipEntry.DEFLATED );
		ZipOutputStream out = new ZipOutputStream( new IdleOutputStream() );
		out.setLevel( COMPRESS_LEVEL );
		out.putNextEntry( entry );
		BufferedInputStream input = new BufferedInputStream(
				new FileInputStream( file ) );
		int b;
		while( (b = input.read()) != -1 ){
			out.write( b );
		}
		input.close();
		out.closeEntry();
		out.close();
		return entry.getCompressedSize();
	}

	/**
	 * 何もしないOutputStream。 ファイルの圧縮後のサイズを調べるために使う。
	 * 
	 * @author s.s.k.
	 */
	private class IdleOutputStream extends OutputStream {

		/*
		 * @see java.io.OutputStream#write(int)
		 */
		public void write( int b ) throws IOException{
		}
	}

	/*
	 * public static void main(String[] args) { try {
	 * UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()); }
	 * catch (Exception e) { e.printStackTrace(); } JFileChooser chooser = new
	 * JFileChooser(); chooser.setMultiSelectionEnabled(true);
	 * chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); if
	 * (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) { File[]
	 * files = chooser.getSelectedFiles(); if (files.length > 0) { ZipFile
	 * zipFile = new ZipFile(); try { zipFile.zip(files, new
	 * File(files[0].getAbsolutePath() + ".zip")); } catch (IOException e) {
	 * e.printStackTrace(); } } } }
	 */
}
